@@ -1,5 +1,5 @@
 /* This file is part of the YAZ toolkit.
- * Copyright (C) 1995-2009 Index Data
+ * Copyright (C) 1995-2010 Index Data
  * See the file LICENSE for details.
  */
 
@@ -33,7 +33,7 @@ static void xml_error_handler(void *ctx, const char *fmt, ...)
     va_start(ap, fmt);
 
     yaz_vsnprintf(buf, sizeof(buf), fmt, ap);
-    yaz_log(YLOG_WARN, "%s: %s", prefix, buf);
+    yaz_log(xml_error_log_level, "%s: %s", prefix, buf);
 
     va_end (ap);
 }
